import Roact from "@rbxts/roact"
import Hooks from "@rbxts/roact-hooks"
import { useWorldContext } from "../contexts/worldContext"
import { ContextActionService, HttpService } from "@rbxts/services"
import Padding from "ReplicatedStorage/ui/padding"

interface slotProps extends Roact.JsxInstanceProperties<Frame> {
    index: number
    keycode: Enum.KeyCode
    tool: Tool

    Event?: Roact.JsxInstanceEvents<Frame>
	Change?: Roact.JsxInstanceChangeEvents<Frame>
}

const slot: Hooks.FC<slotProps> = (props, hooks) => {
    const { useEffect } = hooks
    const { index, keycode, tool } = props

    const spreadableProps = { ...props } as Partial<slotProps>
    delete spreadableProps.index
    delete spreadableProps.keycode
    delete spreadableProps.tool

    const { world, clientState } = useWorldContext(hooks)

    const handleActivated = () =>  {
        tool.Parent !== clientState.character
            ? clientState.character.Humanoid.EquipTool(tool)
            : clientState.character.Humanoid.UnequipTools()
    }

    // maybe opt this into our system for inputs?
    useEffect(() => {
        const guid = HttpService.GenerateGUID(false)
        ContextActionService.BindAction(
            guid,
            (_, userInputState) => {
                if (Enum.UserInputState.Begin === userInputState) handleActivated()
            },
            false,
            keycode
        )
    })

    return (
        <frame {...spreadableProps} BackgroundTransparency={1} SizeConstraint={Enum.SizeConstraint.RelativeYY}>
            <textbutton
                Position={new UDim2(0, 0, 0, 0)}
                Size={new UDim2(1, 0, 1, 0)}
                BackgroundColor3={new Color3()}
                BackgroundTransparency={0.3}
                Text=""
                Event={{
                    Activated: () => {
                        handleActivated()
                    }
                }}
            >
                <uicorner CornerRadius={new UDim(0.05, 0)}/>
                <uistroke
                    Thickness={2}
                    Color={new Color3(0.1, 0.1, 0.1)}
                    ApplyStrokeMode={Enum.ApplyStrokeMode.Border}
                />

                <textlabel
                    Size={new UDim2(1, 0, 0.2, 0)}
					Text={tostring(index)}
					TextXAlignment={Enum.TextXAlignment.Left}
					BackgroundTransparency={1}
					TextColor3={new Color3(1, 1, 1)}
					TextScaled
                >
                    <uistroke Thickness={1}/>
                </textlabel>

                <textlabel
                    Size={new UDim2(1, 0, 1, 0)}
                    AnchorPoint={new Vector2(0.5, 0.5)}
                    Position={new UDim2(0.5, 0, 0.5, 0)}
                    BackgroundTransparency={1}
                    TextScaled
                    Text={tool.Name}
                    TextColor3={Color3.fromRGB(255, 255, 255)}
                >
                    <uistroke Thickness={1}/>
                    <Padding Padding={new UDim(0.05, 0)}/>
                </textlabel>
            </textbutton>
        </frame>
    )
}

export default new Hooks(Roact)(slot)